from qtscreen import *
import serial
import csv
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas
import matplotlib.pyplot as plt
from PyQt6.QtGui import QImage, QPixmap
from datetime import datetime

class MainWindow(QtWidgets.QMainWindow, Ui_MainWindow):
    def __init__(self, *args, **kwargs):
        QtWidgets.QMainWindow.__init__(self, *args, **kwargs)
        self.setupUi(self)
        #Movement between pages
        self.pushButton_2.clicked.connect(self.move_to_chart)
        self.pushButton_3.clicked.connect(self.move_to_output)
        self.pushButton_4.clicked.connect(self.move_to_input)
        self.pushButton_5.clicked.connect(self.move_to_chart)
        self.pushButton_6.clicked.connect(self.move_to_output)
        self.pushButton_7.clicked.connect(self.move_to_input)
        self.pushButton_8.clicked.connect(self.move_to_chart)
        self.pushButton_9.clicked.connect(self.move_to_output)
        self.pushButton_10.clicked.connect(self.move_to_input)
        self.pushButton_13.clicked.connect(self.cicle)
        self.pushButton_12.clicked.connect(self.read_sensor)
        self.horizontalSlider.valueChanged.connect(self.move_sweep)
        self.horizontalSlider_2.valueChanged.connect(self.move_door)
        pixmap = QPixmap("etiquetane.jpg")
        self.label_11.setPixmap(pixmap)

        # Crear un objeto serial
        self.serial = serial.Serial('COM16', 9600)
        self.pushButton.clicked.connect(self.log)

        self.t = []
        self.g = []
        self.chart = chart()
        self.chart.read_data()
        self.chartvl = chartvl(self.t)
       
        self.g1.addWidget(self.chart)
        self.g2.addWidget(self.chartvl)

        self.date = str(self.calendarWidget.selectedDate())
        self.label_3.setText("Complete data untill day: "+ self.date[19:-1])

    #Values from 0 to 100
    def move_sweep(self):
        y = int(self.horizontalSlider.value())
        y *= 6
        y = 'a'+str(y)
        self.serial.write(y.encode())
        
    def move_door(self):
        y = int(self.horizontalSlider_2.value())
        y *= 1.8
        y = 'b'+str(y)
        self.serial.write(y.encode())
        
    def read_sensor(self): # Funcition to see the value of the sensor
        self.serial.write(b'c')
        self.t.append(self.serial.readline())
        self.chartvl = chartvl(self.t)
        self.g2.addWidget(self.chartvl)

    def cicle(self): # Funcition to send the full process
        self.serial.write(b'd')
        p = self.serial.readline()
        current_date = datetime.now().strftime("%d-%m-%Y")
        current_time = datetime.now().strftime("%H:%M")
        for row in self.data_list:
            row['date'] = current_date
            row['time'] = current_time
        fieldnames = self.data_list[0].keys()  # Assuming all rows have the same keys
        with open("data.csv", mode='w', newline='') as file:
            writer = csv.DictWriter(file, fieldnames=fieldnames)
            writer.writeheader()
            for row in self.data_list:
                writer.writerow(row)
        self.chart = chart()
        self.chart.read_data()
        self.g1.addWidget(self.chart)


    def log(self):
        password = 'password'
        passwordtry = self.lineEdit.text()
        if passwordtry == password:
            self.stackedWidget.setCurrentIndex(3)
        else:
            QtWidgets.QMessageBox.warning(self, 'Error', "<font color='black'>Nombre o contraseña Incorrectos</font>")  
    
    def move_to_chart(self):
        self.stackedWidget.setCurrentIndex(3)
    def move_to_input(self):
        self.stackedWidget.setCurrentIndex(1)
    def move_to_output(self):
        self.stackedWidget.setCurrentIndex(0)


    


class chart(FigureCanvas):
    def __init__(self, parent=None):     
        self.fig , self.ax = plt.subplots(1, dpi=100, figsize=(5, 5), sharey=True, facecolor='white')
        super().__init__(self.fig) 
        self.dict = []
        self.date_counts = {}
        self.result = []
        self.dates = []
        self.counts = []
        self.colores = ['red','blue','green','red', 'blue']
       

    def read_data(self):
        with open("data.csv") as file:
            reader = csv.DictReader(file)
            for row in reader:
                self.dict.append(row)
        for entry in self.dict:
            date = entry['Date']
            if date in self.date_counts:
                self.date_counts[date] += 1
            else:
                self.date_counts[date] = 1
        self.result = [{'Date': date, 'Count': count} for date, count in self.date_counts.items()]
        self.dates = [item['Date'] for item in self.result]
        self.counts = [item['Count'] for item in self.result]
        self.ax.bar(self.dates,self.counts, color = self.colores)
        self.fig.suptitle('Times used per day',size=9)

class chartvl(FigureCanvas):
    def __init__(self,vals, parent=None):
        self.fig, self.ax = plt.subplots(1, dpi=100, figsize=(5, 5), facecolor='white')
        super().__init__(self.fig)
        
        self.vals = vals
        self.tamaño = list(range(1, len(vals) + 1))

        # Converting names to integers for scatter plot x-axis values
        x_values = self.tamaño  # Use the same size values for y-axis
        y_values = list(map(int, vals))

        # Create scatter plot
        self.ax.scatter(x_values, y_values, s = self.tamaño)
        self.ax.xaxis.set_major_locator(plt.MaxNLocator(integer=True))
        self.ax.yaxis.set_major_locator(plt.MaxNLocator(integer=True))
        self.fig.suptitle('Scatter Plot', size=9)

if __name__ == '__main__':
    app = QtWidgets.QApplication([])
    window = MainWindow()
    window.show()
    app.exec()